@@ -77,13 +77,13 @@ static void write_callback(struct SoundIoOutStream *outstream, int frame_count_m
         double pitch = 440.0;
         double radians_per_second = pitch * 2.0 * PI;
         for (int frame = 0; frame < frame_count; frame += 1) {
-            double sample = sinf((seconds_offset + frame * seconds_per_frame) * radians_per_second);
+            double sample = sin((seconds_offset + frame * seconds_per_frame) * radians_per_second);
             for (int channel = 0; channel < layout->channel_count; channel += 1) {
                 write_sample(areas[channel].ptr, sample);
                 areas[channel].ptr += areas[channel].step;
             }
         }
-        seconds_offset += seconds_per_frame * frame_count;
+        seconds_offset = fmod(seconds_offset + seconds_per_frame * frame_count, 1.0);
 
         if ((err = soundio_outstream_end_write(outstream))) {
             if (err == SoundIoErrorUnderflow)
@@ -179,7 +179,9 @@ int main(int argc, char **argv) {
         int device_count = soundio_output_device_count(soundio);
         for (int i = 0; i < device_count; i += 1) {
             struct SoundIoDevice *device = soundio_get_output_device(soundio, i);
-            if (strcmp(device->id, device_id) == 0 && device->is_raw == raw) {
+            bool select_this_one = strcmp(device->id, device_id) == 0 && device->is_raw == raw;
+            soundio_device_unref(device);
+            if (select_this_one) {
                 selected_device_index = i;
                 break;
             }
@@ -207,6 +209,11 @@ int main(int argc, char **argv) {
     }
 
     struct SoundIoOutStream *outstream = soundio_outstream_create(device);
+    if (!outstream) {
+        fprintf(stderr, "out of memory\n");
+        return 1;
+    }
+
     outstream->write_callback = write_callback;
     outstream->underflow_callback = underflow_callback;
     outstream->name = stream_name;
