@@ -179,7 +179,7 @@ static int from_cf_string(CFStringRef string_ref, char **out_str, int *out_str_l
     assert(string_ref);
 
     CFIndex length = CFStringGetLength(string_ref);
-    CFIndex max_size = CFStringGetMaximumSizeForEncoding(length, kCFStringEncodingUTF8);
+    CFIndex max_size = CFStringGetMaximumSizeForEncoding(length, kCFStringEncodingUTF8) + 1;
     char *buf = ALLOCATE_NONZERO(char, max_size);
     if (!buf)
         return SoundIoErrorNoMem;
@@ -280,7 +280,10 @@ static int from_coreaudio_layout(const AudioChannelLayout *ca_layout, struct Sou
     switch (ca_layout->mChannelLayoutTag) {
     case kAudioChannelLayoutTag_UseChannelDescriptions:
     {
-        layout->channel_count = ca_layout->mNumberChannelDescriptions;
+        layout->channel_count = soundio_int_min(
+            SOUNDIO_MAX_CHANNELS,
+            ca_layout->mNumberChannelDescriptions
+        );
         for (int i = 0; i < layout->channel_count; i += 1) {
             layout->channels[i] = from_channel_descr(&ca_layout->mChannelDescriptions[i]);
         }
@@ -628,10 +631,15 @@ static int refresh_devices(struct SoundIoPrivate *si) {
 
             rd.device->layout_count = 1;
             rd.device->layouts = &rd.device->current_layout;
-            // in CoreAudio, format is always 32-bit native endian float
-            rd.device->format_count = 1;
-            rd.device->formats = &dev->prealloc_format;
-            rd.device->formats[0] = SoundIoFormatFloat32NE;
+
+            rd.device->format_count = 4;
+            rd.device->formats = ALLOCATE(enum SoundIoFormat, rd.device->format_count);
+            if (!rd.device->formats)
+                return SoundIoErrorNoMem;
+            rd.device->formats[0] = SoundIoFormatS16LE;
+            rd.device->formats[1] = SoundIoFormatS32LE;
+            rd.device->formats[2] = SoundIoFormatFloat32LE;
+            rd.device->formats[3] = SoundIoFormatFloat64LE;
 
             prop_address.mSelector = kAudioDevicePropertyNominalSampleRate;
             prop_address.mScope = aim_to_scope(aim);
@@ -914,6 +922,34 @@ static OSStatus write_callback_ca(void *userdata, AudioUnitRenderActionFlags *io
     return noErr;
 }
 
+static int set_ca_desc(enum SoundIoFormat fmt, AudioStreamBasicDescription *desc) {
+    switch (fmt) {
+    case SoundIoFormatFloat32LE:
+        desc->mFormatFlags = kAudioFormatFlagIsFloat;
+        desc->mBitsPerChannel = 32;
+        break;
+    case SoundIoFormatFloat64LE:
+        desc->mFormatFlags = kAudioFormatFlagIsFloat;
+        desc->mBitsPerChannel = 64;
+        break;
+    case SoundIoFormatS32LE:
+        desc->mFormatFlags = kAudioFormatFlagIsSignedInteger;
+        desc->mBitsPerChannel = 32;
+        break;
+    case SoundIoFormatS16LE:
+        desc->mFormatFlags = kAudioFormatFlagIsSignedInteger;
+        desc->mBitsPerChannel = 16;
+        break;
+	case SoundIoFormatS24LE:
+		desc->mFormatFlags = kAudioFormatFlagIsSignedInteger;
+		desc->mBitsPerChannel = 24;
+		break;
+    default:
+        return SoundIoErrorIncompatibleDevice;
+    }
+    return 0;
+}
+
 static int outstream_open_ca(struct SoundIoPrivate *si, struct SoundIoOutStreamPrivate *os) {
     struct SoundIoOutStreamCoreAudio *osca = &os->backend_data.coreaudio;
     struct SoundIoOutStream *outstream = &os->pub;
@@ -954,12 +990,15 @@ static int outstream_open_ca(struct SoundIoPrivate *si, struct SoundIoOutStreamP
     AudioStreamBasicDescription format = {0};
     format.mSampleRate = outstream->sample_rate;
     format.mFormatID = kAudioFormatLinearPCM;
-    format.mFormatFlags = kAudioFormatFlagIsFloat;
+    int err;
+    if ((err = set_ca_desc(outstream->format, &format))) {
+        outstream_destroy_ca(si, os);
+        return err;
+    }
     format.mBytesPerPacket = outstream->bytes_per_frame;
     format.mFramesPerPacket = 1;
     format.mBytesPerFrame = outstream->bytes_per_frame;
     format.mChannelsPerFrame = outstream->layout.channel_count;
-    format.mBitsPerChannel = 32;
 
     if ((os_err = AudioUnitSetProperty(osca->instance, kAudioOutputUnitProperty_CurrentDevice,
         kAudioUnitScope_Input, OUTPUT_ELEMENT, &dca->device_id, sizeof(AudioDeviceID))))
@@ -972,7 +1011,7 @@ static int outstream_open_ca(struct SoundIoPrivate *si, struct SoundIoOutStreamP
         kAudioUnitScope_Input, OUTPUT_ELEMENT, &format, sizeof(AudioStreamBasicDescription))))
     {
         outstream_destroy_ca(si, os);
-        return SoundIoErrorOpeningDevice;
+        return SoundIoErrorIncompatibleDevice;
     }
 
     AURenderCallbackStruct render_callback = {write_callback_ca, os};
@@ -1006,6 +1045,11 @@ static int outstream_open_ca(struct SoundIoPrivate *si, struct SoundIoOutStreamP
         return SoundIoErrorOpeningDevice;
     }
 
+	if ((os_err = AudioUnitGetParameter (osca->instance, kHALOutputParam_Volume, kAudioUnitScope_Global, 0, &outstream->volume))) {
+		outstream_destroy_ca(si, os);
+		return SoundIoErrorOpeningDevice;
+	}
+
     osca->hardware_latency = dca->latency_frames / (double)outstream->sample_rate;
 
     return 0;
@@ -1076,6 +1120,18 @@ static int outstream_get_latency_ca(struct SoundIoPrivate *si, struct SoundIoOut
     return 0;
 }
 
+static int outstream_set_volume_ca(struct SoundIoPrivate *si, struct SoundIoOutStreamPrivate *os, float volume) {
+    struct SoundIoOutStreamCoreAudio *osca = &os->backend_data.coreaudio;
+    struct SoundIoOutStream *outstream = &os->pub;
+
+    OSStatus os_err;
+    if ((os_err = AudioUnitSetParameter (osca->instance, kHALOutputParam_Volume, kAudioUnitScope_Global, 0, volume, 0))) {
+        return SoundIoErrorIncompatibleDevice;
+    }
+    outstream->volume = volume;
+    return 0;
+}
+
 static OSStatus on_instream_device_overload(AudioObjectID in_object_id, UInt32 in_number_addresses,
     const AudioObjectPropertyAddress in_addresses[], void *in_client_data)
 {
@@ -1134,7 +1190,7 @@ static OSStatus read_callback_ca(void *userdata, AudioUnitRenderActionFlags *io_
         assert(audio_buffer->mNumberChannels == instream->layout.channel_count);
         assert(audio_buffer->mDataByteSize == in_number_frames * instream->bytes_per_frame);
         for (int ch = 0; ch < instream->layout.channel_count; ch += 1) {
-            isca->areas[ch].ptr = ((char*)audio_buffer->mData) + instream->bytes_per_sample;
+            isca->areas[ch].ptr = ((char*)audio_buffer->mData) + (instream->bytes_per_sample * ch);
             isca->areas[ch].step = instream->bytes_per_frame;
         }
     } else {
@@ -1245,12 +1301,16 @@ static int instream_open_ca(struct SoundIoPrivate *si, struct SoundIoInStreamPri
     AudioStreamBasicDescription format = {0};
     format.mSampleRate = instream->sample_rate;
     format.mFormatID = kAudioFormatLinearPCM;
-    format.mFormatFlags = kAudioFormatFlagIsFloat;
     format.mBytesPerPacket = instream->bytes_per_frame;
     format.mFramesPerPacket = 1;
     format.mBytesPerFrame = instream->bytes_per_frame;
     format.mChannelsPerFrame = instream->layout.channel_count;
-    format.mBitsPerChannel = 32;
+
+    int err;
+    if ((err = set_ca_desc(instream->format, &format))) {
+        instream_destroy_ca(si, is);
+        return err;
+    }
 
     if ((os_err = AudioUnitSetProperty(isca->instance, kAudioUnitProperty_StreamFormat,
         kAudioUnitScope_Output, INPUT_ELEMENT, &format, sizeof(AudioStreamBasicDescription))))
@@ -1413,6 +1473,7 @@ int soundio_coreaudio_init(struct SoundIoPrivate *si) {
     si->outstream_clear_buffer = outstream_clear_buffer_ca;
     si->outstream_pause = outstream_pause_ca;
     si->outstream_get_latency = outstream_get_latency_ca;
+    si->outstream_set_volume = outstream_set_volume_ca;
 
     si->instream_open = instream_open_ca;
     si->instream_destroy = instream_destroy_ca;
