@@ -5,6 +5,10 @@
  * See http://opensource.org/licenses/MIT
  */
 
+#ifndef WIN32_LEAN_AND_MEAN
+#define WIN32_LEAN_AND_MEAN
+#endif
+
 #define INITGUID
 #define CINTERFACE
 #define COBJMACROS
@@ -21,6 +25,11 @@
 
 #include <stdio.h>
 
+// Some HRESULT values are not defined by the windows headers
+#ifndef E_NOTFOUND
+#define E_NOTFOUND 0x80070490
+#endif //E_NOTFOUND
+
 #ifdef __cplusplus
 // In C++ mode, IsEqualGUID() takes its arguments by reference
 #define IS_EQUAL_GUID(a, b) IsEqualGUID(*(a), *(b))
@@ -34,6 +43,7 @@
 #define IID_IAUDIORENDERCLIENT                (IID_IAudioRenderClient)
 #define IID_IMMDEVICEENUMERATOR               (IID_IMMDeviceEnumerator)
 #define IID_IAUDIOCAPTURECLIENT               (IID_IAudioCaptureClient)
+#define IID_ISIMPLEAUDIOVOLUME                (IID_ISimpleAudioVolume)
 #define CLSID_MMDEVICEENUMERATOR              (CLSID_MMDeviceEnumerator)
 #define PKEY_DEVICE_FRIENDLYNAME              (PKEY_Device_FriendlyName)
 #define PKEY_AUDIOENGINE_DEVICEFORMAT         (PKEY_AudioEngine_DeviceFormat)
@@ -76,6 +86,10 @@ static const IID IID_IAudioCaptureClient = {
     //MIDL_INTERFACE("C8ADBD64-E71E-48a0-A4DE-185C395CD317")
     0xc8adbd64, 0xe71e, 0x48a0, {0xa4, 0xde, 0x18, 0x5c, 0x39, 0x5c, 0xd3, 0x17}
 };
+static const IID IID_ISimpleAudioVolume = {
+    //MIDL_INTERFACE("87ce5498-68d6-44e5-9215-6da47ef883d8")
+    0x87ce5498, 0x68d6, 0x44e5,{ 0x92, 0x15, 0x6d, 0xa4, 0x7e, 0xf8, 0x83, 0xd8 }
+};
 
 #else
 #define IS_EQUAL_GUID(a, b) IsEqualGUID((a), (b))
@@ -91,6 +105,7 @@ static const IID IID_IAudioCaptureClient = {
 #define IID_IAUDIORENDERCLIENT (&IID_IAudioRenderClient)
 #define IID_IMMDEVICEENUMERATOR (&IID_IMMDeviceEnumerator)
 #define IID_IAUDIOCAPTURECLIENT (&IID_IAudioCaptureClient)
+#define IID_ISIMPLEAUDIOVOLUME (&IID_ISimpleAudioVolume)
 #endif
 
 // Attempting to use the Windows-supplied versions of these constants resulted
@@ -662,40 +677,59 @@ static int refresh_devices(struct SoundIoPrivate *si) {
     if (FAILED(hr = IMMDeviceEnumerator_GetDefaultAudioEndpoint(siw->device_enumerator, eRender,
                     eMultimedia, &rd.default_render_device)))
     {
-        deinit_refresh_devices(&rd);
-        return SoundIoErrorOpeningDevice;
+        if(hr != E_NOTFOUND) {
+            deinit_refresh_devices(&rd);
+            if(hr == E_OUTOFMEMORY) {
+                return SoundIoErrorNoMem;
+            }
+            return SoundIoErrorOpeningDevice;
+        }
     }
-    if (rd.lpwstr) {
-        CoTaskMemFree(rd.lpwstr);
-        rd.lpwstr = NULL;
-    }
-    if (FAILED(hr = IMMDevice_GetId(rd.default_render_device, &rd.lpwstr))) {
-        deinit_refresh_devices(&rd);
-        return SoundIoErrorOpeningDevice;
-    }
-    if ((err = from_lpwstr(rd.lpwstr, &rd.default_render_id, &rd.default_render_id_len))) {
-        deinit_refresh_devices(&rd);
-        return err;
+    if(rd.default_render_device) {
+        if (rd.lpwstr) {
+            CoTaskMemFree(rd.lpwstr);
+            rd.lpwstr = NULL;
+        }
+        if (FAILED(hr = IMMDevice_GetId(rd.default_render_device, &rd.lpwstr))) {
+            deinit_refresh_devices(&rd);
+            // MSDN states the IMMDevice_GetId can fail if the device is NULL, or if we're out of memory
+            // We know the device point isn't NULL so we're necessarily out of memory
+            return SoundIoErrorNoMem;
+        }
+        if ((err = from_lpwstr(rd.lpwstr, &rd.default_render_id, &rd.default_render_id_len))) {
+            deinit_refresh_devices(&rd);
+            return err;
+        }
     }
 
 
     if (FAILED(hr = IMMDeviceEnumerator_GetDefaultAudioEndpoint(siw->device_enumerator, eCapture,
                     eMultimedia, &rd.default_capture_device)))
     {
-        deinit_refresh_devices(&rd);
-        return SoundIoErrorOpeningDevice;
+        if(hr != E_NOTFOUND) {
+            deinit_refresh_devices(&rd);
+            if(hr == E_OUTOFMEMORY) {
+                return SoundIoErrorNoMem;
+            }
+            return SoundIoErrorOpeningDevice;
+        }
     }
-    if (rd.lpwstr) {
-        CoTaskMemFree(rd.lpwstr);
-        rd.lpwstr = NULL;
-    }
-    if (FAILED(hr = IMMDevice_GetId(rd.default_capture_device, &rd.lpwstr))) {
-        deinit_refresh_devices(&rd);
-        return SoundIoErrorOpeningDevice;
-    }
-    if ((err = from_lpwstr(rd.lpwstr, &rd.default_capture_id, &rd.default_capture_id_len))) {
-        deinit_refresh_devices(&rd);
-        return err;
+    if(rd.default_capture_device) {
+        if (rd.lpwstr) {
+            CoTaskMemFree(rd.lpwstr);
+            rd.lpwstr = NULL;
+        }
+        if (FAILED(hr = IMMDevice_GetId(rd.default_capture_device, &rd.lpwstr))) {
+            deinit_refresh_devices(&rd);
+            if(hr == E_OUTOFMEMORY) {
+                return SoundIoErrorNoMem;
+            }
+            return SoundIoErrorOpeningDevice;
+        }
+        if ((err = from_lpwstr(rd.lpwstr, &rd.default_capture_id, &rd.default_capture_id_len))) {
+            deinit_refresh_devices(&rd);
+            return err;
+        }
     }
 
 
@@ -703,11 +737,16 @@ static int refresh_devices(struct SoundIoPrivate *si) {
                     eAll, DEVICE_STATE_ACTIVE, &rd.collection)))
     {
         deinit_refresh_devices(&rd);
+        if(hr == E_OUTOFMEMORY) {
+            return SoundIoErrorNoMem;
+        }
         return SoundIoErrorOpeningDevice;
     }
 
     UINT unsigned_count;
     if (FAILED(hr = IMMDeviceCollection_GetCount(rd.collection, &unsigned_count))) {
+        // In theory this shouldn't happen since the only documented failure case is that
+        // rd.collection is NULL, but then EnumAudioEndpoints should have failed.
         deinit_refresh_devices(&rd);
         return SoundIoErrorOpeningDevice;
     }
@@ -732,16 +771,14 @@ static int refresh_devices(struct SoundIoPrivate *si) {
             rd.mm_device = NULL;
         }
         if (FAILED(hr = IMMDeviceCollection_Item(rd.collection, device_i, &rd.mm_device))) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
+            continue;
         }
         if (rd.lpwstr) {
             CoTaskMemFree(rd.lpwstr);
             rd.lpwstr = NULL;
         }
         if (FAILED(hr = IMMDevice_GetId(rd.mm_device, &rd.lpwstr))) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
+            continue;
         }
 
 
@@ -786,177 +823,30 @@ static int refresh_devices(struct SoundIoPrivate *si) {
             return SoundIoErrorNoMem;
         }
 
-        if (rd.audio_client) {
-            IUnknown_Release(rd.audio_client);
-            rd.audio_client = NULL;
-        }
-        if (FAILED(hr = IMMDevice_Activate(rd.mm_device, IID_IAUDIOCLIENT,
-                        CLSCTX_ALL, NULL, (void**)&rd.audio_client)))
-        {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
-        }
-
-        REFERENCE_TIME default_device_period;
-        REFERENCE_TIME min_device_period;
-        if (FAILED(hr = IAudioClient_GetDevicePeriod(rd.audio_client,
-                        &default_device_period, &min_device_period)))
-        {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
-        }
-        dev_w_shared->period_duration = from_reference_time(default_device_period);
-        rd.device_shared->software_latency_current = dev_w_shared->period_duration;
-
-        dev_w_raw->period_duration = from_reference_time(min_device_period);
-        rd.device_raw->software_latency_min = dev_w_raw->period_duration * 2;
-
-
         if (rd.endpoint) {
             IMMEndpoint_Release(rd.endpoint);
             rd.endpoint = NULL;
         }
         if (FAILED(hr = IMMDevice_QueryInterface(rd.mm_device, IID_IMMENDPOINT, (void**)&rd.endpoint))) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_raw->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+            rd.device_raw = NULL;
+            continue;
         }
 
         EDataFlow data_flow;
         if (FAILED(hr = IMMEndpoint_GetDataFlow(rd.endpoint, &data_flow))) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_raw->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+            rd.device_raw = NULL;
+            continue;
         }
 
         rd.device_shared->aim = data_flow_to_aim(data_flow);
         rd.device_raw->aim = rd.device_shared->aim;
 
-        if (rd.prop_store) {
-            IPropertyStore_Release(rd.prop_store);
-            rd.prop_store = NULL;
-        }
-        if (FAILED(hr = IMMDevice_OpenPropertyStore(rd.mm_device, STGM_READ, &rd.prop_store))) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
-        }
-
-        if (rd.prop_variant_value_inited) {
-            PropVariantClear(&rd.prop_variant_value);
-            rd.prop_variant_value_inited = false;
-        }
-        PropVariantInit(&rd.prop_variant_value);
-        rd.prop_variant_value_inited = true;
-        if (FAILED(hr = IPropertyStore_GetValue(rd.prop_store,
-                        PKEY_DEVICE_FRIENDLYNAME, &rd.prop_variant_value)))
-        {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
-        }
-        if (!rd.prop_variant_value.pwszVal) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
-        }
-        int device_name_len;
-        if ((err = from_lpwstr(rd.prop_variant_value.pwszVal, &rd.device_shared->name, &device_name_len))) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
-        }
-
-        rd.device_raw->name = soundio_str_dupe(rd.device_shared->name, device_name_len);
-        if (!rd.device_raw->name) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorNoMem;
-        }
-
-        // Get the format that WASAPI opens the device with for shared streams.
-        // This is guaranteed to work, so we use this to modulate the sample
-        // rate while holding the format constant and vice versa.
-        if (rd.prop_variant_value_inited) {
-            PropVariantClear(&rd.prop_variant_value);
-            rd.prop_variant_value_inited = false;
-        }
-        PropVariantInit(&rd.prop_variant_value);
-        rd.prop_variant_value_inited = true;
-        if (FAILED(hr = IPropertyStore_GetValue(rd.prop_store, PKEY_AUDIOENGINE_DEVICEFORMAT,
-                        &rd.prop_variant_value)))
-        {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
-        }
-        WAVEFORMATEXTENSIBLE *valid_wave_format = (WAVEFORMATEXTENSIBLE *)rd.prop_variant_value.blob.pBlobData;
-        if (valid_wave_format->Format.wFormatTag != WAVE_FORMAT_EXTENSIBLE) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
-        }
-        if ((err = detect_valid_sample_rates(&rd, valid_wave_format, dev_raw,
-                        AUDCLNT_SHAREMODE_EXCLUSIVE)))
-        {
-            deinit_refresh_devices(&rd);
-            return err;
-        }
-        if ((err = detect_valid_formats(&rd, valid_wave_format, dev_raw,
-                        AUDCLNT_SHAREMODE_EXCLUSIVE)))
-        {
-            deinit_refresh_devices(&rd);
-            return err;
-        }
-
-        if (rd.wave_format) {
-            CoTaskMemFree(rd.wave_format);
-            rd.wave_format = NULL;
-        }
-        if (FAILED(hr = IAudioClient_GetMixFormat(rd.audio_client, (WAVEFORMATEX**)&rd.wave_format))) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
-        }
-        if (rd.wave_format->Format.wFormatTag != WAVE_FORMAT_EXTENSIBLE) {
-            deinit_refresh_devices(&rd);
-            return SoundIoErrorOpeningDevice;
-        }
-        rd.device_shared->sample_rate_current = rd.wave_format->Format.nSamplesPerSec;
-        rd.device_shared->current_format = from_wave_format_format(rd.wave_format);
-
-
-        if (rd.device_shared->aim == SoundIoDeviceAimOutput) {
-            // For output streams in shared mode,
-            // WASAPI performs resampling, so any value is valid.
-            // Let's pick some reasonable min and max values.
-            rd.device_shared->sample_rate_count = 1;
-            rd.device_shared->sample_rates = &dev_shared->prealloc_sample_rate_range;
-            rd.device_shared->sample_rates[0].min = soundio_int_min(SOUNDIO_MIN_SAMPLE_RATE,
-                    rd.device_shared->sample_rate_current);
-            rd.device_shared->sample_rates[0].max = soundio_int_max(SOUNDIO_MAX_SAMPLE_RATE,
-                    rd.device_shared->sample_rate_current);
-        } else {
-            // Shared mode input stream: mix format is all we can do.
-            rd.device_shared->sample_rate_count = 1;
-            rd.device_shared->sample_rates = &dev_shared->prealloc_sample_rate_range;
-            rd.device_shared->sample_rates[0].min = rd.device_shared->sample_rate_current;
-            rd.device_shared->sample_rates[0].max = rd.device_shared->sample_rate_current;
-        }
-
-        if ((err = detect_valid_formats(&rd, rd.wave_format, dev_shared,
-                        AUDCLNT_SHAREMODE_SHARED)))
-        {
-            deinit_refresh_devices(&rd);
-            return err;
-        }
-
-        from_wave_format_layout(rd.wave_format, &rd.device_shared->current_layout);
-        rd.device_shared->layout_count = 1;
-        rd.device_shared->layouts = &rd.device_shared->current_layout;
-
-        if ((err = detect_valid_layouts(&rd, valid_wave_format, dev_raw,
-                        AUDCLNT_SHAREMODE_EXCLUSIVE)))
-        {
-            deinit_refresh_devices(&rd);
-            return err;
-        }
-
-        IMMDevice_AddRef(rd.mm_device);
-        dev_w_shared->mm_device = rd.mm_device;
-        dev_w_raw->mm_device = rd.mm_device;
-        rd.mm_device = NULL;
-
         struct SoundIoListDevicePtr *device_list;
         if (rd.device_shared->aim == SoundIoDeviceAimOutput) {
             device_list = &rd.devices_info->output_devices;
@@ -979,12 +869,192 @@ static int refresh_devices(struct SoundIoPrivate *si) {
             deinit_refresh_devices(&rd);
             return err;
         }
-        rd.device_shared = NULL;
-
         if ((err = SoundIoListDevicePtr_append(device_list, rd.device_raw))) {
             deinit_refresh_devices(&rd);
             return err;
         }
+
+        if (rd.audio_client) {
+            IUnknown_Release(rd.audio_client);
+            rd.audio_client = NULL;
+        }
+        if (FAILED(hr = IMMDevice_Activate(rd.mm_device, IID_IAUDIOCLIENT,
+                        CLSCTX_ALL, NULL, (void**)&rd.audio_client)))
+        {
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_raw->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+            rd.device_raw = NULL;
+            continue;
+        }
+
+        REFERENCE_TIME default_device_period;
+        REFERENCE_TIME min_device_period;
+        if (FAILED(hr = IAudioClient_GetDevicePeriod(rd.audio_client,
+                        &default_device_period, &min_device_period)))
+        {
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_raw->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+            rd.device_raw = NULL;
+            continue;
+        }
+        dev_w_shared->period_duration = from_reference_time(default_device_period);
+        rd.device_shared->software_latency_current = dev_w_shared->period_duration;
+
+        dev_w_raw->period_duration = from_reference_time(min_device_period);
+        rd.device_raw->software_latency_min = dev_w_raw->period_duration * 2;
+
+        if (rd.prop_store) {
+            IPropertyStore_Release(rd.prop_store);
+            rd.prop_store = NULL;
+        }
+        if (FAILED(hr = IMMDevice_OpenPropertyStore(rd.mm_device, STGM_READ, &rd.prop_store))) {
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_raw->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+            rd.device_raw = NULL;
+            continue;
+        }
+
+        if (rd.prop_variant_value_inited) {
+            PropVariantClear(&rd.prop_variant_value);
+            rd.prop_variant_value_inited = false;
+        }
+        PropVariantInit(&rd.prop_variant_value);
+        rd.prop_variant_value_inited = true;
+        if (FAILED(hr = IPropertyStore_GetValue(rd.prop_store,
+                        PKEY_DEVICE_FRIENDLYNAME, &rd.prop_variant_value)))
+        {
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_raw->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+            rd.device_raw = NULL;
+            continue;
+        }
+        if (!rd.prop_variant_value.pwszVal) {
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_raw->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+            rd.device_raw = NULL;
+            continue;
+        }
+        int device_name_len;
+        if ((err = from_lpwstr(rd.prop_variant_value.pwszVal, &rd.device_shared->name, &device_name_len))) {
+            rd.device_shared->probe_error = err;
+            rd.device_raw->probe_error = err;
+            rd.device_shared = NULL;
+            rd.device_raw = NULL;
+            continue;
+        }
+
+        rd.device_raw->name = soundio_str_dupe(rd.device_shared->name, device_name_len);
+        if (!rd.device_raw->name) {
+            deinit_refresh_devices(&rd);
+            return SoundIoErrorNoMem;
+        }
+
+        // Get the format that WASAPI opens the device with for shared streams.
+        // This is guaranteed to work, so we use this to modulate the sample
+        // rate while holding the format constant and vice versa.
+        if (rd.prop_variant_value_inited) {
+            PropVariantClear(&rd.prop_variant_value);
+            rd.prop_variant_value_inited = false;
+        }
+        PropVariantInit(&rd.prop_variant_value);
+        rd.prop_variant_value_inited = true;
+        if (FAILED(hr = IPropertyStore_GetValue(rd.prop_store, PKEY_AUDIOENGINE_DEVICEFORMAT,
+                        &rd.prop_variant_value)))
+        {
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_raw->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+            rd.device_raw = NULL;
+            continue;
+        }
+        WAVEFORMATEXTENSIBLE *valid_wave_format = (WAVEFORMATEXTENSIBLE *)rd.prop_variant_value.blob.pBlobData;
+        if (valid_wave_format->Format.wFormatTag != WAVE_FORMAT_EXTENSIBLE) {
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_raw->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+            rd.device_raw = NULL;
+            continue;
+        }
+        if ((err = detect_valid_sample_rates(&rd, valid_wave_format, dev_raw,
+                        AUDCLNT_SHAREMODE_EXCLUSIVE)))
+        {
+            rd.device_raw->probe_error = err;
+            rd.device_raw = NULL;
+        }
+        if (rd.device_raw && (err = detect_valid_formats(&rd, valid_wave_format, dev_raw,
+                        AUDCLNT_SHAREMODE_EXCLUSIVE)))
+        {
+            rd.device_raw->probe_error = err;
+            rd.device_raw = NULL;
+        }
+        if (rd.device_raw && (err = detect_valid_layouts(&rd, valid_wave_format, dev_raw,
+            AUDCLNT_SHAREMODE_EXCLUSIVE)))
+        {
+            rd.device_raw->probe_error = err;
+            rd.device_raw = NULL;
+        }
+
+        if (rd.wave_format) {
+            CoTaskMemFree(rd.wave_format);
+            rd.wave_format = NULL;
+        }
+        if (FAILED(hr = IAudioClient_GetMixFormat(rd.audio_client, (WAVEFORMATEX**)&rd.wave_format))) {
+            // According to MSDN GetMixFormat only applies to shared-mode devices.
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+        }
+        else if(rd.wave_format && (rd.wave_format->Format.wFormatTag != WAVE_FORMAT_EXTENSIBLE)) {
+            rd.device_shared->probe_error = SoundIoErrorOpeningDevice;
+            rd.device_shared = NULL;
+        }
+
+        if(rd.device_shared) {
+            rd.device_shared->sample_rate_current = rd.wave_format->Format.nSamplesPerSec;
+            rd.device_shared->current_format = from_wave_format_format(rd.wave_format);
+
+            if (rd.device_shared->aim == SoundIoDeviceAimOutput) {
+                // For output streams in shared mode,
+                // WASAPI performs resampling, so any value is valid.
+                // Let's pick some reasonable min and max values.
+                rd.device_shared->sample_rate_count = 1;
+                rd.device_shared->sample_rates = &dev_shared->prealloc_sample_rate_range;
+                rd.device_shared->sample_rates[0].min = soundio_int_min(SOUNDIO_MIN_SAMPLE_RATE,
+                    rd.device_shared->sample_rate_current);
+                rd.device_shared->sample_rates[0].max = soundio_int_max(SOUNDIO_MAX_SAMPLE_RATE,
+                    rd.device_shared->sample_rate_current);
+            }
+            else {
+                // Shared mode input stream: mix format is all we can do.
+                rd.device_shared->sample_rate_count = 1;
+                rd.device_shared->sample_rates = &dev_shared->prealloc_sample_rate_range;
+                rd.device_shared->sample_rates[0].min = rd.device_shared->sample_rate_current;
+                rd.device_shared->sample_rates[0].max = rd.device_shared->sample_rate_current;
+            }
+
+            if ((err = detect_valid_formats(&rd, rd.wave_format, dev_shared,
+                AUDCLNT_SHAREMODE_SHARED)))
+            {
+                rd.device_shared->probe_error = err;
+                rd.device_shared = NULL;
+            }
+            else {
+                from_wave_format_layout(rd.wave_format, &rd.device_shared->current_layout);
+                rd.device_shared->layout_count = 1;
+                rd.device_shared->layouts = &rd.device_shared->current_layout;
+            }
+        }
+
+        IMMDevice_AddRef(rd.mm_device);
+        dev_w_shared->mm_device = rd.mm_device;
+        dev_w_raw->mm_device = rd.mm_device;
+        rd.mm_device = NULL;
+
+        rd.device_shared = NULL;
         rd.device_raw = NULL;
     }
 
@@ -1117,6 +1187,8 @@ static void force_device_scan_wasapi(struct SoundIoPrivate *si) {
 static void outstream_thread_deinit(struct SoundIoPrivate *si, struct SoundIoOutStreamPrivate *os) {
     struct SoundIoOutStreamWasapi *osw = &os->backend_data.wasapi;
 
+    if (osw->audio_volume_control)
+        IUnknown_Release(osw->audio_volume_control);
     if (osw->audio_render_client)
         IUnknown_Release(osw->audio_render_client);
     if (osw->audio_session_control)
@@ -1196,11 +1268,11 @@ static int outstream_do_open(struct SoundIoPrivate *si, struct SoundIoOutStreamP
         if (FAILED(hr = IAudioClient_GetMixFormat(osw->audio_client, (WAVEFORMATEX **)&mix_format))) {
             return SoundIoErrorOpeningDevice;
         }
-        wave_format.Format.nSamplesPerSec = mix_format->Format.nSamplesPerSec;
+        wave_format.Format.nSamplesPerSec = (DWORD)outstream->sample_rate;
+        osw->need_resample = (mix_format->Format.nSamplesPerSec != wave_format.Format.nSamplesPerSec);
         CoTaskMemFree(mix_format);
         mix_format = NULL;
-        osw->need_resample = (wave_format.Format.nSamplesPerSec != (DWORD)outstream->sample_rate);
-        flags = osw->need_resample ? AUDCLNT_STREAMFLAGS_RATEADJUST : 0;
+        flags = osw->need_resample ? AUDCLNT_STREAMFLAGS_AUTOCONVERTPCM | AUDCLNT_STREAMFLAGS_SRC_DEFAULT_QUALITY : 0;
         share_mode = AUDCLNT_SHAREMODE_SHARED;
         periodicity = 0;
         buffer_duration = to_reference_time(4.0);
@@ -1228,11 +1300,11 @@ static int outstream_do_open(struct SoundIoPrivate *si, struct SoundIoOutStreamP
                 if (FAILED(hr = IAudioClient_GetMixFormat(osw->audio_client, (WAVEFORMATEX **)&mix_format))) {
                     return SoundIoErrorOpeningDevice;
                 }
-                wave_format.Format.nSamplesPerSec = mix_format->Format.nSamplesPerSec;
+                wave_format.Format.nSamplesPerSec = (DWORD)outstream->sample_rate;
+                osw->need_resample = (mix_format->Format.nSamplesPerSec != wave_format.Format.nSamplesPerSec);
                 CoTaskMemFree(mix_format);
                 mix_format = NULL;
-                osw->need_resample = (wave_format.Format.nSamplesPerSec != (DWORD)outstream->sample_rate);
-                flags = osw->need_resample ? AUDCLNT_STREAMFLAGS_RATEADJUST : 0;
+                flags = osw->need_resample ? AUDCLNT_STREAMFLAGS_AUTOCONVERTPCM | AUDCLNT_STREAMFLAGS_SRC_DEFAULT_QUALITY : 0;
                 to_wave_format_layout(&outstream->layout, &wave_format);
                 to_wave_format_format(outstream->format, &wave_format);
                 complete_wave_format_data(&wave_format);
@@ -1277,17 +1349,6 @@ static int outstream_do_open(struct SoundIoPrivate *si, struct SoundIoOutStreamP
         if (FAILED(hr = IAudioClient_SetEventHandle(osw->audio_client, osw->h_event))) {
             return SoundIoErrorOpeningDevice;
         }
-    } else if (osw->need_resample) {
-        if (FAILED(hr = IAudioClient_GetService(osw->audio_client, IID_IAUDIOCLOCKADJUSTMENT,
-                        (void**)&osw->audio_clock_adjustment)))
-        {
-            return SoundIoErrorOpeningDevice;
-        }
-        if (FAILED(hr = IAudioClockAdjustment_SetSampleRate(osw->audio_clock_adjustment,
-                        outstream->sample_rate)))
-        {
-            return SoundIoErrorOpeningDevice;
-        }
     }
 
     if (outstream->name) {
@@ -1314,6 +1375,17 @@ static int outstream_do_open(struct SoundIoPrivate *si, struct SoundIoOutStreamP
         return SoundIoErrorOpeningDevice;
     }
 
+    if (FAILED(hr = IAudioClient_GetService(osw->audio_client, IID_ISIMPLEAUDIOVOLUME,
+                    (void **)&osw->audio_volume_control)))
+    {
+        return SoundIoErrorOpeningDevice;
+    }
+
+    if (FAILED(hr = osw->audio_volume_control->lpVtbl->GetMasterVolume(osw->audio_volume_control, &outstream->volume)))
+    {
+        return SoundIoErrorOpeningDevice;
+    }
+
     return 0;
 }
 
@@ -1642,6 +1714,21 @@ static int outstream_get_latency_wasapi(struct SoundIoPrivate *si, struct SoundI
     return 0;
 }
 
+static int outstream_set_volume_wasapi(struct SoundIoPrivate *si, struct SoundIoOutStreamPrivate *os, float volume)
+{
+    struct SoundIoOutStream *outstream = &os->pub;
+    struct SoundIoOutStreamWasapi *osw = &os->backend_data.wasapi;
+
+    HRESULT hr;
+    if (FAILED(hr = osw->audio_volume_control->lpVtbl->SetMasterVolume(osw->audio_volume_control, volume, NULL)))
+    {
+        return SoundIoErrorIncompatibleDevice;
+    }
+
+    outstream->volume = volume;
+    return 0;
+}
+
 static void instream_thread_deinit(struct SoundIoPrivate *si, struct SoundIoInStreamPrivate *is) {
     struct SoundIoInStreamWasapi *isw = &is->backend_data.wasapi;
 
@@ -1784,6 +1871,10 @@ static int instream_do_open(struct SoundIoPrivate *si, struct SoundIoInStreamPri
     if (FAILED(hr = IAudioClient_GetBufferSize(isw->audio_client, &isw->buffer_frame_count))) {
         return SoundIoErrorOpeningDevice;
     }
+    if (instream->software_latency == 0.0)
+        instream->software_latency = 1.0;
+    instream->software_latency = soundio_double_clamp(device->software_latency_min,
+            instream->software_latency, device->software_latency_max);
     if (isw->is_raw)
         instream->software_latency = isw->buffer_frame_count / (double)instream->sample_rate;
 
@@ -2016,7 +2107,9 @@ static int instream_begin_read_wasapi(struct SoundIoPrivate *si, struct SoundIoI
         {
             return SoundIoErrorStreaming;
         }
-        isw->read_buf_frames_left = frames_to_read;
+		isw->opened_buf_frames = frames_to_read;
+		isw->read_buf_frames_left = frames_to_read;
+
         if (flags & AUDCLNT_BUFFERFLAGS_SILENT)
             isw->read_buf = NULL;
     }
@@ -2028,6 +2121,8 @@ static int instream_begin_read_wasapi(struct SoundIoPrivate *si, struct SoundIoI
         for (int ch = 0; ch < instream->layout.channel_count; ch += 1) {
             isw->areas[ch].ptr = isw->read_buf + ch * instream->bytes_per_sample;
             isw->areas[ch].step = instream->bytes_per_frame;
+
+			isw->areas[ch].ptr += instream->bytes_per_frame * (isw->opened_buf_frames - isw->read_buf_frames_left);
         }
 
         *out_areas = isw->areas;
@@ -2041,10 +2136,15 @@ static int instream_begin_read_wasapi(struct SoundIoPrivate *si, struct SoundIoI
 static int instream_end_read_wasapi(struct SoundIoPrivate *si, struct SoundIoInStreamPrivate *is) {
     struct SoundIoInStreamWasapi *isw = &is->backend_data.wasapi;
     HRESULT hr;
-    if (FAILED(hr = IAudioCaptureClient_ReleaseBuffer(isw->audio_capture_client, isw->read_frame_count))) {
-        return SoundIoErrorStreaming;
-    }
-    isw->read_buf_frames_left -= isw->read_frame_count;
+
+	isw->read_buf_frames_left -= isw->read_frame_count;
+
+	if (isw->read_buf_frames_left <= 0) {
+		if (FAILED(hr = IAudioCaptureClient_ReleaseBuffer(isw->audio_capture_client, isw->opened_buf_frames))) {
+			return SoundIoErrorStreaming;
+		}
+	}
+
     return 0;
 }
 
@@ -2218,6 +2318,7 @@ int soundio_wasapi_init(struct SoundIoPrivate *si) {
     si->outstream_clear_buffer = outstream_clear_buffer_wasapi;
     si->outstream_pause = outstream_pause_wasapi;
     si->outstream_get_latency = outstream_get_latency_wasapi;
+    si->outstream_set_volume = outstream_set_volume_wasapi;
 
     si->instream_open = instream_open_wasapi;
     si->instream_destroy = instream_destroy_wasapi;
